@@ -276,6 +276,8 @@ int Wsrep_high_priority_service::append_fragment_and_commit(
   m_thd->wsrep_cs().after_applying();
   m_thd->mdl_context.release_transactional_locks();
 
+  free_root(m_thd->mem_root, MYF(MY_KEEP_PREALLOC));
+
   thd_proc_info(m_thd, "wsrep applier committed");
 
   DBUG_RETURN(ret);
@@ -334,6 +336,8 @@ int Wsrep_high_priority_service::commit(const wsrep::ws_handle& ws_handle,
 
   thd->lex->sql_command= SQLCOM_END;
 
+  free_root(thd->mem_root, MYF(MY_KEEP_PREALLOC));
+
   must_exit_= check_exit_status();
   DBUG_RETURN(ret);
 }
@@ -346,6 +350,9 @@ int Wsrep_high_priority_service::rollback(const wsrep::ws_handle& ws_handle,
   int ret= (trans_rollback_stmt(m_thd) || trans_rollback(m_thd));
   m_thd->mdl_context.release_transactional_locks();
   m_thd->mdl_context.release_explicit_locks();
+
+  free_root(m_thd->mem_root, MYF(MY_KEEP_PREALLOC));
+
   DBUG_RETURN(ret);
 }
 
@@ -453,11 +460,15 @@ Wsrep_applier_service::Wsrep_applier_service(THD* thd)
   thd->wsrep_cs().open(wsrep::client_id(thd->thread_id));
   thd->wsrep_cs().before_command();
   thd->wsrep_cs().debug_log_level(wsrep_debug);
-
+  if (!thd->slave_thread)
+    thd->system_thread_info.rpl_sql_info=
+      new rpl_sql_thread_info(thd->wsrep_rgi->rli->mi->rpl_filter);
 }
 
 Wsrep_applier_service::~Wsrep_applier_service()
 {
+  if (!m_thd->slave_thread)
+    delete m_thd->system_thread_info.rpl_sql_info;
   m_thd->wsrep_cs().after_command_before_result();
   m_thd->wsrep_cs().after_command_after_result();
   m_thd->wsrep_cs().close();
